@@ -8,7 +8,7 @@
 #   The GNU General Public License, Version 3, June 2007
 # 
 package Games::Pandemic::Tk::Utils;
-our $VERSION = '0.4.0';
+our $VERSION = '0.5.0';
 
 # ABSTRACT: Tk utilities for gui building
 
@@ -28,6 +28,7 @@ our @EXPORT = qw{
     @FILLX  @FILLY  @FILL2
     @XFILLX @XFILLY @XFILL2
     @PAD1   @PAD10 @PAD20 @PADX10
+    @IPAD10
     @ENON   @ENOFF
     image   pandemic_icon
 };
@@ -54,6 +55,9 @@ our @PAD10   = ( -padx => 10, -pady => 10);
 our @PAD20   = ( -padx => 20, -pady => 20);
 our @PADX10  = ( -padx => 10 );
 
+# internal padding
+our @IPAD10   = ( -ipadx => 10, -ipady => 10);
+
 # enabled state
 our @ENON    = ( -state => 'normal' );
 our @ENOFF   = ( -state => 'disabled' );
@@ -92,7 +96,7 @@ Games::Pandemic::Tk::Utils - Tk utilities for gui building
 
 =head1 VERSION
 
-version 0.4.0
+version 0.5.0
 
 =head1 DESCRIPTION
 
